/*
 * pam_pwexport.c -- hook to export passwords to a standalone exe
 * Copyright (C) 2000 Peter Samuelson <peter@cadcamlab.org>
 *
 * Bits stolen from other Linux-PAM modules
 * Copyright (C) 1996-1998 Andrew Morgan <morgan@transmeta.com>
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define PAM_SM_AUTH
#define PAM_SM_PASSWORD
#include <security/pam_modules.h>

/*
 * Find the executable name.
 * Simple enough ... look for the leading /
 */
static char *find_exe (int argc, const char **argv)
{
  while (argc) {
    if (**argv == '/')
      return (char *)*argv;
    argc--; argv++;
  }
  return NULL;
}

/* FIXME should use fork directly! */
static FILE *my_popenwrite(const char *exe)
{
  return popen(exe, "w");
}

/* FIXME should use wait directly! */
static int my_pclose(FILE *f)
{
  return pclose(f);
}

/********************************************************************/
/* The functional part of the module.  Called for authentication. */
PAM_EXTERN int pam_sm_authenticate (pam_handle_t *pamh, int flags UNUSED, int argc, const char **argv)
{
  char *user, *tok;
  FILE *out;
  char *exe = find_exe (argc, argv);

  pam_get_item (pamh, PAM_USER, (const void **)&user);
  pam_get_item (pamh, PAM_AUTHTOK, (const void **)&tok);
  if (!user || !tok || !exe)
    return PAM_SUCCESS;

  if ((out=my_popenwrite(exe))) {
    fprintf(out, "user %s\npassword %s\n", user, tok);
    my_pclose(out);
  }

  return PAM_SUCCESS;
}

/* Required to be defined, but not required to be useful... */
PAM_EXTERN int pam_sm_setcred (pam_handle_t *pamh UNUSED, int flags UNUSED,
		int argc UNUSED, const char **argv UNUSED)
{
  return PAM_SUCCESS;
}


/********************************************************************/
/* Changing passwords. */
PAM_EXTERN int pam_sm_chauthtok (pam_handle_t *pamh, int flags UNUSED,
		int argc, const char **argv)
{
  char *user, *tok, *oldtok;
  FILE *out;
  char *exe = find_exe (argc, argv);

  pam_get_item (pamh, PAM_USER, (const void **)&user);
  pam_get_item (pamh, PAM_OLDAUTHTOK, (const void **)&oldtok);
  pam_get_item (pamh, PAM_AUTHTOK, (const void **)&tok);
  if (!user || !tok || !exe)
    return PAM_SUCCESS;

  if ((out = my_popenwrite(exe))) {
    if (oldtok)
      fprintf(out, "user %s\noldpassword %s\npassword %s\n", user, oldtok, tok);
    else
      fprintf(out, "user %s\npassword %s\n", user, tok);
    my_pclose(out);
  }

  return PAM_SUCCESS;
}


/********************************************************************/
/* In case you want to compile this into libpam itself. */
#ifdef PAM_STATIC
struct pam_module _pam_deny_modstruct = {
  "pam_pwexport",
  pam_sm_authenticate,
  pam_sm_setcred,
  NULL,
  NULL,
  NULL,
  pam_sm_chauthtok,
};
#endif
